/*******************************************************************************
*   Author       -  Kiraly Tibor                                               *
*                   http://www.tkiraaly.hu                                     *
*   Date         -  2012.12.09.                                                *
*   Chip         -  Atmel ATtiny2313 & HD77480                                 *
*   Compiler     -  avr-gcc (WinAVR )                                          *
*                                                                              *
*   Bekapcsolas bekapcsolja a LED-et,majd az LCD-re kiirja: Szia Levi!         *
*                                                                              *
********************************************************************************
*   PonyProg Configuration and Security Bits (bepipalva):                      *
*   Belso 4MHz-es osszc.:     SUT0, CKSEL3, CKSEL2, CKSEL0                     *
*                                                                              *
*******************************************************************************/
//
//
//    LCD bekotese:
//
//    14   LCD_D7       - PB3 - 15
//    13   LCD_D6       - PB2 - 14
//    12   LCD_D5       - PB1 - 13
//    11   LCD_D4       - PB0 - 12
//    10   LCD_D3       - GND
//     9   LCD_D2       - GND
//     8   LCD_D1       - GND
//     7   LCD_D0       - GND
//     6   LCD_E        - PD6 - 11   (1 - change data - 0)
//     5   LCD_RW       - GND        (0- write, 1- read)
//     4   LCD_RS       - PD5        (0- istruction, 1- data)
//     3   LCD_KONTR    - 10 kOhm trim.
//     2   LCD_VDD      - +5V
//     1   LCD_VSS      - GND


#ifndef F_CPU
#define F_CPU                4000000             //MHz
#endif



#include "tkiraaly_attiny2313.h"
#include <util/delay.h>




void lcd_init4( void);
void lcd_putc( UC);




#define LCD_E                6
#define LCD_E_0              BC( PORTD, LCD_E)
#define LCD_E_1              BS( PORTD, LCD_E)

#define LCD_RS               5
#define LCD_RS_UTASITAS      BC( PORTD, LCD_RS)
#define LCD_RS_ADAT          BS( PORTD, LCD_RS)

#define LED                  4
#define LED_BE               BC( PORTD, LED)
#define LED_KI               BS( PORTD, LED)

#define LCD_PORT             PORTB               // also 4 bit
#define LCD_PORT_ENABLE      DDRB= 0B00001111    // also 4 bit kimenet




int main( void)
{
   DDRD= 0B01110000;                             // kimenetek beallitasa (1)
   LCD_PORT_ENABLE;
   lcd_init4();
   LED_BE;
   lcd_putc( 'S');
   lcd_putc( 'z');
   lcd_putc( 'i');
   lcd_putc( 'a');
   lcd_putc( ' ');
   lcd_putc( 'L');
   lcd_putc( 'e');
   lcd_putc( 'v');
   lcd_putc( 'i');
   lcd_putc( '!');
   return 0;
}




void lcd_init4( void)                            // LCD init - 4 bit interface
{
   _delay_ms( 50);                               // 50 msec varakozas
   LCD_PORT= 0x00000010;
   LCD_E_1;
   LCD_E_0;
   _delay_ms( 5);
   LCD_E_1;
   LCD_E_0;
   _delay_ms( 1);
   lcd_putc( 0B00101000);                        // 4 bit interface, 2 sor, 5x8 pontos betu
   lcd_putc( 0B00101000);                        // megegszer, az LCD miatt
   lcd_putc( 0B00001100);                        // kijelzes be, cursor ki
   lcd_putc( 0B00000110);                        // set entry mode
   lcd_putc( 0B00000010);                        // bal felso sarokba
   _delay_ms( 2);
   LCD_RS_ADAT;
}




void lcd_putc( UC c)                             // LCD 1 betu kiiras
{
   LCD_E_1;                                      // felso 4 bit
   LCD_PORT= ( LCD_PORT & 0B11110000) | ( 0B00001111 & c >> 4);
   LCD_E_0;
   LCD_E_1;                                      // also 4 bit
   LCD_PORT= ( LCD_PORT & 0B11110000) | ( 0B00001111 & c);
   LCD_E_0;
   _delay_us( 37);
}
